/*
 * Copyright (C) 2008-2013 J.Rios <anonbeat@gmail.com>
 * Copyright (C) 2013 Jörn Magens
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file LICENSE.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth 
 * Floor, Boston, MA  02110-1301  USA
 * https://www.gnu.org/licenses/lgpl-2.1.txt
 *
 * Author:
 * 	Jörn Magens <shuerhaaken@googlemail.com>
 * 	Matias De lellis <mati86dl@gmail.com>
 * 	Pavel Vasin <rat4vier@gmail.com>
 */


#ifndef TAGINFO_H
#define TAGINFO_H

#include <stdio.h>
#include <string.h>

#include <tag.h>
#include <fileref.h>
#include <asffile.h>
#include <mp4file.h>
#include <oggfile.h>
#include <xiphcomment.h>
#include <aifffile.h>
#include <wavfile.h>
#include <speexfile.h>

#include <apetag.h>
#include <apefile.h>
#include <id3v2tag.h>


using namespace TagLib;
using namespace std;



namespace TagInfo {
    
    
    enum MediaFileType {
        MEDIA_FILE_TYPE_UNKNOWN,
        MEDIA_FILE_TYPE_AAC,
        MEDIA_FILE_TYPE_AIF,
        MEDIA_FILE_TYPE_APE,
        MEDIA_FILE_TYPE_ASF,
        MEDIA_FILE_TYPE_FLAC,
        MEDIA_FILE_TYPE_M4A,
        MEDIA_FILE_TYPE_M4B,
        MEDIA_FILE_TYPE_M4P,
        MEDIA_FILE_TYPE_MP3,
        MEDIA_FILE_TYPE_MP4,
        MEDIA_FILE_TYPE_MPC,
        MEDIA_FILE_TYPE_OGA,
        MEDIA_FILE_TYPE_OGG,
        MEDIA_FILE_TYPE_TTA,
        MEDIA_FILE_TYPE_WAV,
        MEDIA_FILE_TYPE_WMA,
        MEDIA_FILE_TYPE_WV,
        MEDIA_FILE_TYPE_SPEEX,
        MEDIA_FILE_TYPE_WMV
    };
    
    enum ImageType {
        IMAGE_TYPE_UNKNOWN,
        IMAGE_TYPE_JPEG,
        IMAGE_TYPE_PNG
    };
    
    
    class Info {
        protected :
            TagLib::FileRef * taglib_fileref;
            TagLib::File *    taglib_file;
            TagLib::Tag *     taglib_tag;
            
            String file_name;
            
            //Tags
            String title;
            String genre;
            String artist;
            String album_artist;
            String album;
            String composer;
            String comments;
            int    tracknumber;
            int    year;
            bool   is_compilation;
            
            int disk_number;
            
            
            //Audioproperties
            int length_seconds;
            int bitrate;
            int samplerate;
            int channels;
            
            //Extras
            int playcount;
            int rating;
            StringList track_labels;
            String track_labels_string;
            StringList artist_labels;
            String artist_labels_string;
            StringList album_labels;
            String album_labels_string;
            
            int changedflag;
            bool has_image;
            bool valid;
            
            
            void set_file_name(const String &filename);
            void read_virtual_tags(TagLib::Tag * tag);
            void write_virtual_tags(TagLib::Tag * tag);
            
            virtual bool create_file_ref();
            
            
        public:
            
            //General stuff
            String get_file_name(void);
            
            bool is_valid();
            
            //Tag access functions
            String get_title(void) const;
            void   set_title(String new_title);
            
            String get_genre(void) const;
            void   set_genre(String new_genre);
            
            String get_artist(void) const;
            void   set_artist(String new_artist);
            
            String get_album_artist(void) const;
            void   set_album_artist(String new_artist);
            
            String get_album(void) const;
            void   set_album(String new_album);
            
            String get_composer(void) const;
            void   set_composer(String new_composer);
            
            String get_comments(void) const;
            void   set_comments(String new_comments);
            
            int  get_tracknumber() const;
            void set_tracknumber(int new_tracknumber);
            
            int  get_year() const;
            void set_year(int new_year);
            
            //Extra
            int  get_playcount() const;
            void set_playcount(int new_playcount);
            
            /*rating 1 - 5 ;  0 -> not set*/
            int  get_rating() const;
            void set_rating(int new_rating);
            
            int  get_disk_number() const;
            void set_disk_number(int number);
            
            bool get_has_image() const;
            
            //Audio properties access functions
            int get_length_seconds() const;
            int get_bitrate() const;
            int get_channels() const;
            int get_samplerate() const;
           
            
            //Labels
            StringList get_track_labels_list() const;
            void set_track_labels_list(const StringList &new_track_labels_list);
            StringList get_artist_labels_list() const;
            void set_artist_labels_list(const StringList &new_artist_labels_list);
            StringList get_album_labels_list() const;
            void set_album_labels_list(const StringList &new_album_labels_list);
            
            
            //Misc
            bool get_is_compilation() const;
            void set_is_compilation(bool compilation);
            
            
            Info(const String &filename = "", bool create_own_fileref = false);
            ~Info();
            
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
            
            static Info * create_tag_info(const String &file);
            static Info * create_tag_info_with_format(const String &file, MediaFileType format);
            static Info * create_tag_info_from_mime(const String &filename, const String &mime_type);
    };
    
    
    class Mp3Info : public Info {
        protected :
            ID3v2::Tag * taglib_tagId3v2;
            virtual bool create_file_ref();
            
        public :
            Mp3Info(const String &filename = "");
            ~Mp3Info();
            
            virtual bool read(void);
            virtual bool write();
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class FlacInfo : public Info {
        protected :
            Ogg::XiphComment * xiphcomment;
            virtual bool create_file_ref();
        
        public :
            FlacInfo(const String &filename = "");
            ~FlacInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class OggInfo : public Info {
        protected :
            Ogg::XiphComment * xiphcomment;
            virtual bool create_file_ref();
            
        public :
            OggInfo(const String &filename = "");
            ~OggInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class SpeexInfo : public Info {
        protected :
            Ogg::XiphComment * xiphcomment;
            virtual bool create_file_ref();
            
        public :
            SpeexInfo(const String &filename = "");
            ~SpeexInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class Mp4Info : public Info {
        protected :
            TagLib::MP4::Tag * mp4_tag;
            virtual bool create_file_ref();
            
        public :
            Mp4Info(const String &filename = "");
            ~Mp4Info();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class ApeInfo : public Info {
        protected:
            TagLib::APE::Tag * taglib_apetag;
            virtual bool create_file_ref();
            
        public:
            ApeInfo(const String &filename = "");
            ~ApeInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
    };
    
    
    class MpcInfo : public Info {
        protected :
            TagLib::APE::Tag * taglib_apetag;
            virtual bool create_file_ref();
            
        public :
            MpcInfo(const String &filename = "");
            ~MpcInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
    };
    
    
    class WavPackInfo : public Info {
        protected :
            TagLib::APE::Tag * taglib_apetag;
            virtual bool create_file_ref();
            
        public :
            WavPackInfo(const String &filename = "");
            ~WavPackInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class TrueAudioInfo : public Info {
        protected :
            ID3v2::Tag * taglib_tagId3v2;
            virtual bool create_file_ref();
        
        public :
            TrueAudioInfo(const String &filename = "");
            ~TrueAudioInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class AiffInfo : public Info {
        protected :
            ID3v2::Tag * taglib_tagId3v2;
            virtual bool create_file_ref();
            
        public :
            AiffInfo(const String &filename = "");
            ~AiffInfo();
            
            virtual bool read(void);
            virtual bool write();
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class ASFInfo : public Info {
        protected :
            ASF::Tag * asf_tag;
            virtual bool create_file_ref();
        
        public :
            ASFInfo(const String &filename = "");
            ~ASFInfo();
            
            virtual bool read(void);
            virtual bool write(void);
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
    
    
    class WaveInfo : public Info {
        protected :
            ID3v2::Tag * taglib_tagId3v2;
            virtual bool create_file_ref();
            
        public :
            WaveInfo(const String &filename = "");
            ~WaveInfo();
            
            virtual bool read(void);
            virtual bool write();
            
            virtual bool can_handle_images(void) const;
            virtual bool get_image(char*& data, int &data_length, ImageType &image_type) const;
            virtual bool set_image(const char* data, int data_length, ImageType image_type);
            
            virtual bool can_handle_lyrics(void) const;
            virtual String get_lyrics(void) const;
            virtual bool set_lyrics(const String &lyrics);
    };
}
#endif
